@@ -20,9 +20,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307, USA.
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
    The GNU General Public License is contained in the file COPYING.
 */
@@ -64,7 +62,7 @@ static ULong g_max_blocks = 0;
 static ULong g_max_bytes  = 0;
 static ULong g_max_instrs = 0;
 
-// Values for the entire run. Computed at the end.
+// Values for the entire run. Updated each time a block is retired.
 static ULong g_reads_bytes = 0;
 static ULong g_writes_bytes = 0;
 
@@ -618,6 +616,13 @@ void* renew_block ( ThreadId tid, void* p_old, SizeT new_req_szB )
       // New size is smaller or same; block not moved.
       resize_Block(bk->ap, bk->req_szB, new_req_szB);
       bk->req_szB = new_req_szB;
+
+      // Update reads/writes for the implicit copy. Even though we didn't
+      // actually do a copy, we act like we did, to match up with the fact
+      // that we treat this as an additional allocation.
+      bk->reads_bytes += new_req_szB;
+      bk->writes_bytes += new_req_szB;
+
       return p_old;
 
    } else {
@@ -638,15 +643,19 @@ void* renew_block ( ThreadId tid, void* p_old, SizeT new_req_szB )
       // interval tree at the new place.  Do this by removing
       // and re-adding it.
       delete_Block_starting_at( (Addr)p_old );
-      // now 'bk' is no longer in the tree, but the Block itself
-      // is still alive
+      // Now 'bk' is no longer in the tree, but the Block itself
+      // is still alive.
+
+      // Update reads/writes for the copy.
+      bk->reads_bytes += bk->req_szB;
+      bk->writes_bytes += bk->req_szB;
 
       // Update the metadata.
       resize_Block(bk->ap, bk->req_szB, new_req_szB);
       bk->payload = (Addr)p_new;
       bk->req_szB = new_req_szB;
 
-      // and re-add
+      // And re-add it to the interval tree.
       Bool present
          = VG_(addToFM)( interval_tree, (UWord)bk, (UWord)0/*no val*/);
       tl_assert(!present);
@@ -1378,6 +1387,7 @@ static void dh_fini(Int exit_status)
                    VKI_S_IRUSR|VKI_S_IWUSR);
    if (!fp) {
       VG_(umsg)("error: can't open DHAT output file '%s'\n", dhat_out_file);
+      VG_(free)(dhat_out_file);
       return;
    }
 
@@ -1423,6 +1433,7 @@ static void dh_fini(Int exit_status)
       FP(" %c\"%s\"\n", i == 0 ? '[' : ',', json_escape(frames[i]));
    }
    FP(" ]\n");
+   VG_(free)(frames);
 
    FP("}\n");
 
@@ -1452,6 +1463,8 @@ static void dh_fini(Int exit_status)
    VG_(umsg)("  %s\n", dhat_out_file);
    VG_(umsg)("Scroll to the end the displayed page to see a short\n");
    VG_(umsg)("explanation of some of the abbreviations used in the page.\n");
+
+   VG_(free)(dhat_out_file);
 }
 
 //------------------------------------------------------------//
