@@ -1,9 +1,12 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include "memcheck/memcheck.h"
 
 static int a1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789(void)
 {
-  return *(int *)0;
+  int x;
+  (void) VALGRIND_MAKE_MEM_NOACCESS(&x, sizeof(x));
+  return *(int *)&x;
 }
 
 int main(int argc, char **argv)
